@@ -1,6 +1,5 @@
 // -*- mode: cpp; mode: fold -*-
 // Description								/*{{{*/
-// $Id: configuration.cc,v 1.28 2004/04/30 04:00:15 mdz Exp $
 /* ######################################################################
 
    Configuration Class
@@ -19,9 +18,9 @@
 
 #include <apt-pkg/configuration.h>
 #include <apt-pkg/error.h>
-#include <apt-pkg/strutl.h>
 #include <apt-pkg/fileutl.h>
 #include <apt-pkg/macros.h>
+#include <apt-pkg/strutl.h>
 
 #include <ctype.h>
 #include <regex.h>
@@ -31,13 +30,13 @@
 #include <string.h>
 
 #include <algorithm>
-#include <iterator>
-#include <string>
-#include <stack>
-#include <vector>
 #include <fstream>
+#include <iterator>
 #include <sstream>
+#include <stack>
+#include <string>
 #include <unordered_map>
+#include <vector>
 
 #include <apti18n.h>
 
@@ -253,7 +252,7 @@ Configuration::Item *Configuration::Lookup(Item *Head,const char *S,
    if (Len != 0)
    {
       for (; I != 0; Last = &I->Next, I = I->Next)
-	 if ((Res = stringcasecmp(I->Tag,S,S + Len)) == 0)
+	 if (Len == I->Tag.length() && (Res = stringcasecmp(I->Tag,S,S + Len)) == 0)
 	    break;
    }
    else
@@ -729,7 +728,7 @@ bool Configuration::ExistsAny(const char *Name) const
 /* Dump the entire configuration space */
 void Configuration::Dump(ostream& str)
 {
-   Dump(str, NULL, "%f \"%v\";\n", true);
+   Dump(str, NULL, "%F \"%v\";\n", true);
 }
 void Configuration::Dump(ostream& str, char const * const root,
 			 char const * const formatstr, bool const emptyValue)
@@ -840,9 +839,9 @@ bool ReadConfigFile(Configuration &Conf,const string &FName,bool const &AsSectio
 		    unsigned const &Depth)
 {
    // Open the stream for reading
-   ifstream F(FName.c_str(),ios::in);
-   if (F.fail() == true)
-      return _error->Errno("ifstream::ifstream",_("Opening configuration file %s"),FName.c_str());
+   FileFd F;
+   if (OpenConfigurationFileFd(FName, F) == false)
+      return false;
 
    string LineBuffer;
    std::stack<std::string> Stack;
@@ -852,26 +851,15 @@ bool ReadConfigFile(Configuration &Conf,const string &FName,bool const &AsSectio
 
    int CurLine = 0;
    bool InComment = false;
-   while (F.eof() == false)
+   while (F.Eof() == false)
    {
       // The raw input line.
       std::string Input;
+      if (F.ReadLine(Input) == false)
+	 Input.clear();
       // The input line with comments stripped.
       std::string Fragment;
 
-      // Grab the next line of F and place it in Input.
-      do
-	{
-	  char *Buffer = new char[1024];
-
-	  F.clear();
-	  F.getline(Buffer,sizeof(Buffer) / 2);
-
-	  Input += Buffer;
-	  delete[] Buffer;
-	}
-      while (F.fail() && !F.eof());
-
       // Expand tabs in the input line and remove leading and trailing
       // whitespace.
       {
@@ -1161,13 +1149,10 @@ bool ReadConfigFile(Configuration &Conf,const string &FName,bool const &AsSectio
 bool ReadConfigDir(Configuration &Conf,const string &Dir,
 		   bool const &AsSectional, unsigned const &Depth)
 {
-   vector<string> const List = GetListOfFilesInDir(Dir, "conf", true, true);
-
-   // Read the files
-   for (vector<string>::const_iterator I = List.begin(); I != List.end(); ++I)
-      if (ReadConfigFile(Conf,*I,AsSectional,Depth) == false)
-	 return false;
-   return true;
+   bool good = true;
+   for (auto const &I : GetListOfFilesInDir(Dir, "conf", true, true))
+      good = ReadConfigFile(Conf, I, AsSectional, Depth) && good;
+   return good;
 }
 									/*}}}*/
 // MatchAgainstConfig Constructor					/*{{{*/
